@@ -29,7 +29,7 @@
 #pod =for :this This is C<Dist::Zilla::Role::TextTemplater> module documentation. Read this if you want to
 #pod have text templating capabilities in your Dist::Zilla plugin.
 #pod
-#pod =for :that If you are using a C<TextTemplater>-based plugin, read the
+#pod =for :those If you are using a C<TextTemplater>-based plugin, read the
 #pod L<manual|Dist::Zilla::Role::TextTemplater::Manual>. General topics like getting source, building, installing, bug
 #pod reporting and some others are covered in the F<README>.
 #pod
@@ -60,8 +60,8 @@
 #pod
 #pod =head1 DESCRIPTION
 #pod
-#pod The role provides consuming plugin with C<fill_in_string> and C<fill_in_file> methods and bunch of
-#pod accompanying attributes and F<dist.ini> options.
+#pod The role provides a consuming plugin with C<fill_in_string> and C<fill_in_file> methods and bunch
+#pod of accompanying attributes and F<dist.ini> options.
 #pod
 #pod =cut
 
@@ -71,7 +71,7 @@ use Moose::Role;
 use namespace::autoclean;
 
 # ABSTRACT: Have text templating capabilities in your Dist::Zilla plugin
-our $VERSION = '0.006'; # VERSION
+our $VERSION = '0.007'; # VERSION
 
 with 'Dist::Zilla::Role::ErrorLogger' => { -version => 0.005 };
 
@@ -84,10 +84,10 @@ use Text::Template qw{};
 
 #pod =attr delimiters
 #pod
-#pod Pair of open and closing delimiters the code fragments surrounded by.
+#pod Pair of opening delimiter and closing delimiter to denote code fragments in template.
 #pod
 #pod Attribute introduces F<dist.ini> option with the same name. Option value will be split on
-#pod whitespaces (and result should be two words) to initialize the attribute.
+#pod whitespaces (result should be two items) to initialize the attribute.
 #pod
 #pod C<Str|ArrayRef[Str]>, read-only. Default value is C<[ '{{', '}}' ]>.
 #pod
@@ -106,8 +106,8 @@ has delimiters => (
     trigger     => sub {
         my ( $self, $new ) = @_;
         if ( not ref( $new ) ) {
-            $new =~ s{\A\s+}{};     # Drop leading ws, or `split` may leave the first item empty.
-            $new = [ split( qr{\s+}, $new ) ];
+            $new =~ s{\A\s+}{}x;    # Drop leading ws, or `split` may leave the first item empty.
+            $new = [ split( qr{\s+}x, $new ) ];
             @$new == 2
                 or croak "\"delimiters\" value must be Str of *two* whitespace-separated words";
             $self->{ delimiters } = $new;
@@ -146,8 +146,8 @@ has package => (
 #pod
 #pod Attribute introduces F<dist.ini> multi-value option with the same name.
 #pod
-#pod C<ArrayRef[Str]>, read-only, auto dereferenced. Default value is empty array. Another default value
-#pod may be specified by defining C<_build_prepend> method.
+#pod C<ArrayRef[Str]>, read-only, auto dereferenced. Default value is empty array. Consumers may specify
+#pod alternative default by defining C<_build_prepend> method.
 #pod
 #pod See C<PREPEND> option of L<Text::Template/"fill_in">.
 #pod
@@ -168,8 +168,8 @@ sub _build_prepend {
 
 #pod =attr broken
 #pod
-#pod Callback to execute if a code fragment dies. Default callback formats detailed error message and
-#pod log it by calling C<log_error>.
+#pod Callback called if a code fragment dies. Default callback formats detailed error message and sends
+#pod it to the log by calling C<log_error>.
 #pod
 #pod C<CodeRef>, read-only, not an init argument. Default callback may be overridden by defining
 #pod C<_build_broken> method.
@@ -192,8 +192,8 @@ sub _build_broken {
         my $error = $args{ error };     # Error message.
         my $tfile = $self->tt_filename; # Template filename.
         my $tline = $args{ lineno };    # Template line where the problem code fragment begins.
-        my $eline = 0;                                      # Line where the problem occurs,
-        if ( $error =~ m{ at \Q$tfile\E line (\d+)} ) {     # if the problem occurs in template.
+        my $eline = 0;                  # Problem line, if the problem occurs in template.
+        if ( $error =~ m{ \s at \s \Q$tfile\E \s line \s (\d+)}x ) {
             $eline = $1;
         };
         chomp( $error );
@@ -205,7 +205,7 @@ sub _build_broken {
             $tline => 'Code fragment begins in line above',
             $eline => 'Code fragment died in line above',
         );
-        # TODO: Do we need $self->abort() here?
+        return $error;
     };
 };
 
@@ -213,8 +213,8 @@ sub _build_broken {
 
 #pod =attr tt_filename
 #pod
-#pod Name of file being processed, or "template" if no filename was specified (e. g. when processing a
-#pod string and C<filename> option was not specified). Available only during template processing (e. g.
+#pod Name of file being processed, or "template" if no file name was specified (e. g. when processing a
+#pod string and C<filename> option was not specified). Available only during template processing (i. e.
 #pod may be used in a C<broken> subroutine).
 #pod
 #pod C<Str>, read-only, not an init arg.
@@ -231,7 +231,7 @@ has tt_filename => (
 
 #pod =attr tt_template
 #pod
-#pod Template being processed. Available only during template processing (e. g. may be used in a
+#pod Template being processed. Available only during template processing (i. e. may be used in a
 #pod C<broken> subroutine).
 #pod
 #pod C<Str>, read-only, not an init arg.
@@ -248,7 +248,7 @@ has tt_template => (
 
 #pod =attr tt_context
 #pod
-#pod Number of template lines to show above and below error line in error messages.
+#pod Number of template lines to show above and below problem line in error message.
 #pod
 #pod C<Int>, read-only, not an init arg, default value 2.
 #pod
@@ -263,6 +263,43 @@ has tt_context => (
     default     => 2,
 );
 
+# --------------------------------------------------------------------------------------------------
+
+#pod =attr tt_broken_count
+#pod
+#pod Number of C<broken> calls. The counter is increased before C<broken> call.
+#pod
+#pod C<Int>, read-only, not an init arg.
+#pod
+#pod =cut
+
+has tt_broken_count => (
+    is          => 'ro',
+    isa         => 'Int',
+    init_arg    => undef,
+    default     => 0,
+);
+
+# --------------------------------------------------------------------------------------------------
+
+#pod =attr tt_broken_limit
+#pod
+#pod If number of completed C<broken> calls equals or exceeds this limit, processing stops.
+#pod
+#pod C<Int>, read-only, not an init arg, default value 10.
+#pod
+#pod There is no (official) way to change the attribute value now. Let me know if you need it.
+#pod
+#pod =cut
+
+has tt_broken_limit => (
+    is          => 'ro',
+    isa         => 'Int',
+    init_arg    => undef,
+    default     => 10,
+);
+
+
 # --------------------------------------------------------------------------------------------------
 
 #pod =method mvp_multivalue_args
@@ -280,38 +317,86 @@ around mvp_multivalue_args => sub {
 
 #pod =method fill_in_string
 #pod
-#pod     $string = '…';
-#pod     $result = $self->fill_in_string( $string, \%variables, \%extra_args );
-#pod     $result = $self->fill_in_string( $string );
+#pod     $template = '…';
+#pod     $result = $self->fill_in_string( $template, \%variables, \%extra_args );
+#pod     $result = $self->fill_in_string( $template );
 #pod
 #pod The primary method of the role.
 #pod
-#pod The interface is compatible with method of C<TextTemplate> role, so this role can be used as a
-#pod drop-in replacement for C<TextTemplate>. However, methods are implemented slightly differently, it
-#pod may cause subtle difference in behaviour.
-#pod
-#pod Template variables C<$plugin> and C<$dist> are set by default, if not explicitly defined in
-#pod C<\%variables>.
+#pod The C<fill_in_string> interface is compatible with the same-name method of C<TextTemplate> role, so
+#pod this role can be used as a drop-in replacement for C<TextTemplate>. However, methods are
+#pod implemented slightly differently, it may cause subtle difference in behaviour.
 #pod
 #pod The method creates C<Text::Template> object, enforces C<Text::Template> to respect C<filename>
 #pod argument (see L<FILENAME parameter has no
-#pod effect|https://rt.cpan.org/Ticket/Display.html?id=106093>), then calls C<fill_in> method on the
-#pod object. C<$template>, C<\%variables>, and C<package>, C<prepend>, C<broken> attribute values, and
-#pod finally C<%extra_args> are passed to the C<fill_in>.
+#pod effect|https://rt.cpan.org/Ticket/Display.html?id=106093>), takes care about warnings, then calls
+#pod C<fill_in> method on the object, making C<Text::Template> compilation errors (if found) more
+#pod user-friendly.
 #pod
-#pod C<%extra_args> is passed in the end intentionally, so caller may override any of arguments
-#pod specified by the method, for example:
+#pod C<$template> is passed to the C<Text::Template> constructor. C<\%variables>, C<\%extra_args>, and
+#pod C<package>, C<prepend>, C<broken> attributes are combined and passed to both C<Text::Template>
+#pod constructor and C<fill_in> method.
+#pod
+#pod C<\%variables> become C<hash> C<Text::Template> option (see L<Text::Template/"HASH"> for details).
+#pod Variables C<plugin> (reference to object executing the method, i. e. C<$self>) and C<dist>
+#pod (reference to C<Dist::Zilla>, i. e. C<< $self->zilla >>) are added to C<\%variables> automatically,
+#pod if they are not exist.
+#pod
+#pod C<package>, C<prepend>, C<broken> attributes become same-name C<Text::Template> options.
+#pod C<\%extra_args> is expanded to list and passed last, so caller can override any option specified by
+#pod C<fill_in_string>, for example:
 #pod
 #pod     $self->fill_in_string( $template, undef, { package => 'MY' } );
-#pod         #   Execute code fragments in package MY regardless of $self->package value.
 #pod
-#pod It also allows to cancel defining C<$plugin> and C<$dist> variables, e. g.:
+#pod will execute template code fragments in context of C<MY> package regardless of C<package>
+#pod attribute. Another, a bit more complicated example:
 #pod
-#pod     $self->fill_in_string( $template, undef, { hash => {} } );
-#pod         #   Do not define any variables for the template.
+#pod     $self->fill_in_string( $template, undef, { hash => { } } );
+#pod
+#pod processes template with no predefined variables: C<plugin> and C<dist> are added to C<\%variables>,
+#pod but entire C<\%variables> is overridden by C<hash> extra argument.
+#pod
+#pod C<fill_in_string> takes special care about package: by default nested C<fill_in_string> calls use
+#pod the same package as the outermost call. Of course, if C<package> extra argument is explicitly
+#pod specified in inner call, it takes priority over default package.
+#pod
+#pod When defining variables with either C<\%variables> argument or C<hash> extra argument, remember that
+#pod C<Text::Template> dereferences all the references. It especially important if you want to pass
+#pod a reference to template:
+#pod
+#pod     $array = [ … ]; $hash = { … };
+#pod     $self->fill_in_string(
+#pod         $template,
+#pod         { array => \$array, hash => \$hash, plugin => \$self },
+#pod     );
+#pod
+#pod In template, C<$array> will be a reference to array, C<$hash> — reference to hash, C<$plugin> —
+#pod reference to plugin. If you forget to take references, e. g.:
+#pod
+#pod     $self->fill_in_string(
+#pod         $template,
+#pod         { array => $array, hash => $hash, plugin => $self },
+#pod     );
+#pod
+#pod template will have C<@array> (not C<$array>) — array, C<%hash> (not C<$hash>) — hash, and
+#pod C<$plugin> will be (oops!) undefined.
+#pod
+#pod Scalars can be passed either by reference or value:
+#pod
+#pod     $self->fill_in_string(
+#pod         $template,
+#pod         { str1 => "string", str2 => \"string" },
+#pod     );
+#pod
+#pod See L<Text::Template/"HASH"> for more details.
 #pod
 #pod =cut
 
+sub _param($\%) {
+    my ( $name, $args ) = @_;
+    return Text::Template::_param( $name, %$args );     ## no critic ( ProtectPrivateSubs )
+};
+
 sub fill_in_string {
     my ( $self, $string, $hash, $args ) = @_;
     my %hash = (
@@ -322,13 +407,25 @@ sub fill_in_string {
     my %args = (
         hash       => \%hash,
         delimiters => $self->delimiters,
-        $self->package ? ( package => $self->package,              ) : (),
-        $self->prepend ? ( prepend => join( "\n", $self->prepend ) ) : (),
-        $self->broken  ? ( broken  => $self->broken                ) : (),
+        $self->package ? (
+            package => $self->package,
+        ) : (
+        ),
+        $self->prepend ? (
+            prepend => join( "\n", $self->prepend ),
+        ) : (
+        ),
+        $self->broken  ? (
+            broken  => sub {
+                ++ $self->{ tt_broken_count };
+                my $rc = $self->broken->( @_ );
+                return $self->tt_broken_count < $self->tt_broken_limit ? $rc : undef;
+            },
+        ) : (
+        ),
         $args ? %$args : (),
     );
-    my ( $result, $error );
-    {
+    my ( $result, $error ); {
         #   Original version of the code simply called `Text::Template::fill_in_string` function.
         #   However, this trivial approach does not work good because of `Text::Template` bug: it
         #   ignores `filename` argument, see <https://rt.cpan.org/Ticket/Display.html?id=106093>.
@@ -336,10 +433,19 @@ sub fill_in_string {
         #   `Text::Template` object property `FILENAME`. In order to do it, we need access to
         #   `Text::Template` object, that means we have to create it manually, and then call
         #   `fill_in` method on it.
-        local $Text::Template::ERROR;
+        my $filename = ( _param( 'filename', %args ) or 'template' );
+        $self->log_debug( [ 'processing %s', $filename ] );
+        local $Text::Template::ERROR = undef;
         my $tt = Text::Template->new( type => 'STRING', source => $string, %args );
         if ( $tt ) {
-            my $filename = ( Text::Template::_param( 'filename', %args ) or 'template' );
+            my $package = _param( 'package', %args );
+            if ( not defined( $package ) or $package eq '' ) {
+                #   If package was not explicitly specified, create a private package.
+                $package = Dist::Zilla::Role::TextTemplater::_Package->new();
+            };
+            #   Save package (either explicitly specified or auto-generated) in `package`
+            #   attribute, to let recursive `fill_in_string` calls utilize the same package.
+            local $self->{ package     } = "$package";
             local $self->{ tt_filename } = $filename;   # Will be used in `broken`.
             local $self->{ tt_template } = $string;     # ditto.
             $tt->{ FILENAME } = $filename;  # Workaround for the bug.
@@ -348,8 +454,12 @@ sub fill_in_string {
                 chomp( $msg );
                 $self->log( $msg );
             };
-            $result = $tt->fill_in( %args );
-            if ( not defined( $result ) ) {
+            $result = $tt->fill_in( %args, package => "$package" );
+            #   `Text::Template` doc says:
+            #       If the `BROKEN` function returns undef, `Text::Template` will immediately abort
+            #       processing the template and return the text that it has accumulated so far.
+            #   It seems it is not true, `fill_in` returns `undef`, not text accumulated so far.
+            if ( defined( $Text::Template::ERROR ) ) {
                 #   There are few error message which can be generated by "Text::Template":
                 #       Unmatched close brace at line $lineno
                 #       End of data inside program text that began at line $prog_start
@@ -372,13 +482,12 @@ sub fill_in_string {
                         $self->tt_template,
                         $line => "$msg in line above",
                     );
-                    $result = '';   # Define result to avoid double error reporting.
+                    $Text::Template::ERROR = undef;     # Avoid double error reporting.
                 };
             };
         };
-        $error = $Text::Template::ERROR;
-    }
-    if ( not defined( $result ) ) {
+    };
+    if ( defined( $error ) ) {
         $self->log_error( $error );
     };
     $self->abort_if_error();
@@ -450,7 +559,7 @@ sub fill_in_file {
             $args
         );
     };
-    croak "fill_in_string: unsupported file class: $class";
+    croak "fill_in_file: unsupported file class: $class";
 };
 
 # --------------------------------------------------------------------------------------------------
@@ -459,7 +568,8 @@ sub fill_in_file {
 #pod
 #pod C<TextTemplater> uses this method to report errors in templates.
 #pod
-#pod     $self->_show_context( $text,
+#pod     $self->_show_context(
+#pod         $text,
 #pod         $linenum0 => $message0,
 #pod         $linenum1 => $message1,
 #pod         ...
@@ -467,7 +577,7 @@ sub fill_in_file {
 #pod
 #pod C<$text> is a text to show. It could be either a C<Str> (multiline) or C<ArrayRef[Str]> (a line per
 #pod element, either chomped or not). The method does not show entire text, but only problem ("focus")
-#pod lines in context (C<$context> lines above and below each focus line).
+#pod lines in context (C<tt_context> lines above and below each focus line).
 #pod
 #pod Focus lines are specified by pairs C<< $linenum => $message >>, where C<$linenum> is a number of
 #pod problem line, and C<$message> is an error message to print just below the specified line. Multiple
@@ -501,7 +611,7 @@ sub _show_context {
     my $fline  = $indent . '%0' . $w . 'd: %s';
     my $fnote  = $indent . ( ' ' x $w ) . '  ^^^ %s ^^^';               # Notice line format.
     my $fskip  = $indent . ( ' ' x $w ) . '  ...skipped %d lines...';   # "Skipped" notice format.
-    my $last  = 0;                          # Number of the last printed line.
+    my $last   = 0;                             # Number of the last printed line.
     my $show_line = sub {
         my ( $n ) = @_;
         my $line = $text->[ $n - 1 ];
@@ -530,17 +640,49 @@ sub _show_context {
         $last = $n;
     };
     $show_skip->( @$text + 1 );
+    return;
 };
 
 # --------------------------------------------------------------------------------------------------
 
+#   Helper class to create private packages. Actual creation and deletion is implemented in
+#   `Text::Template`, this class just adds automatical package deletion.
+
+{
+    ## no critic ( ProhibitMultiplePackages )
+    package Dist::Zilla::Role::TextTemplater::_Package;
+
+    use strict;
+    use warnings;
+
+    use overload '""' => sub {
+        my ( $self ) = @_;
+        return $self->{ name };
+    };
+
+    sub new {
+        my ( $class ) = @_;
+        my $self = {
+            name => Text::Template::_gensym(),          ## no critic ( ProtectPrivateSubs )
+        };
+        return bless( $self, $class );
+    };
+
+    sub DESTROY {
+        my ( $self ) = @_;
+        Text::Template::_scrubpkg( $self->{ name } );   ## no critic ( ProtectPrivateSubs )
+        return;
+    };
+
+}
+
+# --------------------------------------------------------------------------------------------------
+
 1;
 
 # --------------------------------------------------------------------------------------------------
 
-#pod =head1 NOTES
-#pod
-#pod =head2 C<Text::Template> option spelling
+#pod =note C<Text::Template> option spelling
 #pod
 #pod C<Text::Template> allows a programmer to use different spelling of options: all-caps, first-caps,
 #pod all-lowercase, with and without leading dash, e. g.: C<HASH>, C<Hash>, C<hash>, C<-HASH>, C<-Hash>,
@@ -552,8 +694,8 @@ sub _show_context {
 #pod
 #pod     $self->fill_in_string( $template, undef, { PACKAGE => 'MY' } );
 #pod
-#pod Extra option C<PACKAGE> may or may not not have effect, depending on value of C<package> attribute
-#pod (i. e. presence or absence C<package> option in F<dist.ini> file), because (this is not documented)
+#pod Extra option C<PACKAGE> may or may not have effect, depending on value of C<package> attribute (i.
+#pod e. presence or absence C<package> option in F<dist.ini> file), because (this is not documented)
 #pod spellings are not equal: different spellings have different priority. If C<PACKAGE> and C<package>
 #pod are specified simultaneously, C<package> wins, C<PACKAGE> loses.
 #pod
@@ -561,14 +703,19 @@ sub _show_context {
 #pod and provide your value, use all-lowercase option name. If you want to provide default which can be
 #pod overridden by the user, use all-caps options name.
 #pod
-#pod =head2 C<filename> option
+#pod =cut
+
+# --------------------------------------------------------------------------------------------------
+
+#pod =note C<filename> option
 #pod
-#pod When C<Text::Template> reads template from a file, it uses the file name in error messages, e. g.:
+#pod When C<Text::Template> reads template from a file, it uses the actual file name in error messages,
+#pod e. g.:
 #pod
-#pod     Undefined subroutine &foo called at filename.ext line n
+#pod     Undefined subroutine &foo called at dir/filename.ext line n
 #pod
-#pod where I<filename.ext> is the name of file containing the template. When C<Text::Template> processes
-#pod a string, it uses word "template" instead of file name, e. g.:
+#pod where I<dir/filename.ext> is the name of file containing the template. When C<Text::Template>
+#pod processes a string, it uses word "template" instead of file name, e. g.:
 #pod
 #pod     Undefined subroutine &foo called at template line n
 #pod
@@ -585,7 +732,11 @@ sub _show_context {
 #pod of C<Dist::Zilla::File::OnDisk> class. Thus, using C<filename> option is critical to provide good
 #pod error messages. Actually, C<fill_in_file> implementation looks like
 #pod
-#pod     $file->content( $self->fill_in_string( $file->content, undef, { filename => $file->name } ) );
+#pod     $self->fill_in_string(
+#pod         $file->content,
+#pod         undef,
+#pod         { filename => $file->name },
+#pod     );
 #pod
 #pod There are two problems with the option, though:
 #pod
@@ -604,11 +755,15 @@ sub _show_context {
 #pod I am sure this is a bug and hope it will be fixed eventually. I am afraid it will not be fixed
 #pod soon, though.
 #pod
-#pod Meanwhile, C<Dist-Zilla-Role-TextTemplater> implements a workaround to let the option work, so C<TextTemplater>
+#pod Meanwhile, C<TextTemplater> implements a workaround to let the option work, so C<TextTemplater>
 #pod consumers can utilize the C<filename> option.
 #pod
 #pod =back
 #pod
+#pod =cut
+
+# --------------------------------------------------------------------------------------------------
+
 #pod =head1 SEE ALSO
 #pod
 #pod =for :list
@@ -783,7 +938,7 @@ Dist::Zilla::Role::TextTemplater - Have text templating capabilities in your Dis
 
 =head1 VERSION
 
-Version 0.006, released on 2015-08-13 18:44 UTC.
+Version 0.007, released on 2015-08-30 22:48 UTC.
 
 =head1 WHAT?
 
@@ -825,17 +980,17 @@ reporting and some others are covered in the F<README>.
 
 =head1 DESCRIPTION
 
-The role provides consuming plugin with C<fill_in_string> and C<fill_in_file> methods and bunch of
-accompanying attributes and F<dist.ini> options.
+The role provides a consuming plugin with C<fill_in_string> and C<fill_in_file> methods and bunch
+of accompanying attributes and F<dist.ini> options.
 
 =head1 OBJECT ATTRIBUTES
 
 =head2 delimiters
 
-Pair of open and closing delimiters the code fragments surrounded by.
+Pair of opening delimiter and closing delimiter to denote code fragments in template.
 
 Attribute introduces F<dist.ini> option with the same name. Option value will be split on
-whitespaces (and result should be two words) to initialize the attribute.
+whitespaces (result should be two items) to initialize the attribute.
 
 C<Str|ArrayRef[Str]>, read-only. Default value is C<[ '{{', '}}' ]>.
 
@@ -860,15 +1015,15 @@ Perl code to prepend to the beginning of every code fragment.
 
 Attribute introduces F<dist.ini> multi-value option with the same name.
 
-C<ArrayRef[Str]>, read-only, auto dereferenced. Default value is empty array. Another default value
-may be specified by defining C<_build_prepend> method.
+C<ArrayRef[Str]>, read-only, auto dereferenced. Default value is empty array. Consumers may specify
+alternative default by defining C<_build_prepend> method.
 
 See C<PREPEND> option of L<Text::Template/"fill_in">.
 
 =head2 broken
 
-Callback to execute if a code fragment dies. Default callback formats detailed error message and
-log it by calling C<log_error>.
+Callback called if a code fragment dies. Default callback formats detailed error message and sends
+it to the log by calling C<log_error>.
 
 C<CodeRef>, read-only, not an init argument. Default callback may be overridden by defining
 C<_build_broken> method.
@@ -877,27 +1032,41 @@ See C<BROKEN> option of L<Text::Template/"fill_in">.
 
 =head2 tt_filename
 
-Name of file being processed, or "template" if no filename was specified (e. g. when processing a
-string and C<filename> option was not specified). Available only during template processing (e. g.
+Name of file being processed, or "template" if no file name was specified (e. g. when processing a
+string and C<filename> option was not specified). Available only during template processing (i. e.
 may be used in a C<broken> subroutine).
 
 C<Str>, read-only, not an init arg.
 
 =head2 tt_template
 
-Template being processed. Available only during template processing (e. g. may be used in a
+Template being processed. Available only during template processing (i. e. may be used in a
 C<broken> subroutine).
 
 C<Str>, read-only, not an init arg.
 
 =head2 tt_context
 
-Number of template lines to show above and below error line in error messages.
+Number of template lines to show above and below problem line in error message.
 
 C<Int>, read-only, not an init arg, default value 2.
 
 There is no (official) way to change the attribute value now. Let me know if you need it.
 
+=head2 tt_broken_count
+
+Number of C<broken> calls. The counter is increased before C<broken> call.
+
+C<Int>, read-only, not an init arg.
+
+=head2 tt_broken_limit
+
+If number of completed C<broken> calls equals or exceeds this limit, processing stops.
+
+C<Int>, read-only, not an init arg, default value 10.
+
+There is no (official) way to change the attribute value now. Let me know if you need it.
+
 =head1 OBJECT METHODS
 
 =head2 mvp_multivalue_args
@@ -906,35 +1075,78 @@ The method tells C<Dist::Zilla> that C<prepend> is a multi-value option.
 
 =head2 fill_in_string
 
-    $string = '…';
-    $result = $self->fill_in_string( $string, \%variables, \%extra_args );
-    $result = $self->fill_in_string( $string );
+    $template = '…';
+    $result = $self->fill_in_string( $template, \%variables, \%extra_args );
+    $result = $self->fill_in_string( $template );
 
 The primary method of the role.
 
-The interface is compatible with method of C<TextTemplate> role, so this role can be used as a
-drop-in replacement for C<TextTemplate>. However, methods are implemented slightly differently, it
-may cause subtle difference in behaviour.
-
-Template variables C<$plugin> and C<$dist> are set by default, if not explicitly defined in
-C<\%variables>.
+The C<fill_in_string> interface is compatible with the same-name method of C<TextTemplate> role, so
+this role can be used as a drop-in replacement for C<TextTemplate>. However, methods are
+implemented slightly differently, it may cause subtle difference in behaviour.
 
 The method creates C<Text::Template> object, enforces C<Text::Template> to respect C<filename>
 argument (see L<FILENAME parameter has no
-effect|https://rt.cpan.org/Ticket/Display.html?id=106093>), then calls C<fill_in> method on the
-object. C<$template>, C<\%variables>, and C<package>, C<prepend>, C<broken> attribute values, and
-finally C<%extra_args> are passed to the C<fill_in>.
+effect|https://rt.cpan.org/Ticket/Display.html?id=106093>), takes care about warnings, then calls
+C<fill_in> method on the object, making C<Text::Template> compilation errors (if found) more
+user-friendly.
 
-C<%extra_args> is passed in the end intentionally, so caller may override any of arguments
-specified by the method, for example:
+C<$template> is passed to the C<Text::Template> constructor. C<\%variables>, C<\%extra_args>, and
+C<package>, C<prepend>, C<broken> attributes are combined and passed to both C<Text::Template>
+constructor and C<fill_in> method.
+
+C<\%variables> become C<hash> C<Text::Template> option (see L<Text::Template/"HASH"> for details).
+Variables C<plugin> (reference to object executing the method, i. e. C<$self>) and C<dist>
+(reference to C<Dist::Zilla>, i. e. C<< $self->zilla >>) are added to C<\%variables> automatically,
+if they are not exist.
+
+C<package>, C<prepend>, C<broken> attributes become same-name C<Text::Template> options.
+C<\%extra_args> is expanded to list and passed last, so caller can override any option specified by
+C<fill_in_string>, for example:
 
     $self->fill_in_string( $template, undef, { package => 'MY' } );
-        #   Execute code fragments in package MY regardless of $self->package value.
 
-It also allows to cancel defining C<$plugin> and C<$dist> variables, e. g.:
+will execute template code fragments in context of C<MY> package regardless of C<package>
+attribute. Another, a bit more complicated example:
 
-    $self->fill_in_string( $template, undef, { hash => {} } );
-        #   Do not define any variables for the template.
+    $self->fill_in_string( $template, undef, { hash => { } } );
+
+processes template with no predefined variables: C<plugin> and C<dist> are added to C<\%variables>,
+but entire C<\%variables> is overridden by C<hash> extra argument.
+
+C<fill_in_string> takes special care about package: by default nested C<fill_in_string> calls use
+the same package as the outermost call. Of course, if C<package> extra argument is explicitly
+specified in inner call, it takes priority over default package.
+
+When defining variables with either C<\%variables> argument or C<hash> extra argument, remember that
+C<Text::Template> dereferences all the references. It especially important if you want to pass
+a reference to template:
+
+    $array = [ … ]; $hash = { … };
+    $self->fill_in_string(
+        $template,
+        { array => \$array, hash => \$hash, plugin => \$self },
+    );
+
+In template, C<$array> will be a reference to array, C<$hash> — reference to hash, C<$plugin> —
+reference to plugin. If you forget to take references, e. g.:
+
+    $self->fill_in_string(
+        $template,
+        { array => $array, hash => $hash, plugin => $self },
+    );
+
+template will have C<@array> (not C<$array>) — array, C<%hash> (not C<$hash>) — hash, and
+C<$plugin> will be (oops!) undefined.
+
+Scalars can be passed either by reference or value:
+
+    $self->fill_in_string(
+        $template,
+        { str1 => "string", str2 => \"string" },
+    );
+
+See L<Text::Template/"HASH"> for more details.
 
 =head2 fill_in_file
 
@@ -961,7 +1173,8 @@ C<Dist::Zilla::Role::MutableFile>) file content is also updated.
 
 C<TextTemplater> uses this method to report errors in templates.
 
-    $self->_show_context( $text,
+    $self->_show_context(
+        $text,
         $linenum0 => $message0,
         $linenum1 => $message1,
         ...
@@ -969,7 +1182,7 @@ C<TextTemplater> uses this method to report errors in templates.
 
 C<$text> is a text to show. It could be either a C<Str> (multiline) or C<ArrayRef[Str]> (a line per
 element, either chomped or not). The method does not show entire text, but only problem ("focus")
-lines in context (C<$context> lines above and below each focus line).
+lines in context (C<tt_context> lines above and below each focus line).
 
 Focus lines are specified by pairs C<< $linenum => $message >>, where C<$linenum> is a number of
 problem line, and C<$message> is an error message to print just below the specified line. Multiple
@@ -991,8 +1204,8 @@ have subtle consequences. Let us consider an example:
 
     $self->fill_in_string( $template, undef, { PACKAGE => 'MY' } );
 
-Extra option C<PACKAGE> may or may not not have effect, depending on value of C<package> attribute
-(i. e. presence or absence C<package> option in F<dist.ini> file), because (this is not documented)
+Extra option C<PACKAGE> may or may not have effect, depending on value of C<package> attribute (i.
+e. presence or absence C<package> option in F<dist.ini> file), because (this is not documented)
 spellings are not equal: different spellings have different priority. If C<PACKAGE> and C<package>
 are specified simultaneously, C<package> wins, C<PACKAGE> loses.
 
@@ -1002,12 +1215,13 @@ overridden by the user, use all-caps options name.
 
 =head2 C<filename> option
 
-When C<Text::Template> reads template from a file, it uses the file name in error messages, e. g.:
+When C<Text::Template> reads template from a file, it uses the actual file name in error messages,
+e. g.:
 
-    Undefined subroutine &foo called at filename.ext line n
+    Undefined subroutine &foo called at dir/filename.ext line n
 
-where I<filename.ext> is the name of file containing the template. When C<Text::Template> processes
-a string, it uses word "template" instead of file name, e. g.:
+where I<dir/filename.ext> is the name of file containing the template. When C<Text::Template>
+processes a string, it uses word "template" instead of file name, e. g.:
 
     Undefined subroutine &foo called at template line n
 
@@ -1024,7 +1238,11 @@ C<Text::Template> always processes strings and never reads files, because readin
 of C<Dist::Zilla::File::OnDisk> class. Thus, using C<filename> option is critical to provide good
 error messages. Actually, C<fill_in_file> implementation looks like
 
-    $file->content( $self->fill_in_string( $file->content, undef, { filename => $file->name } ) );
+    $self->fill_in_string(
+        $file->content,
+        undef,
+        { filename => $file->name },
+    );
 
 There are two problems with the option, though:
 
@@ -1043,7 +1261,7 @@ C<Text::Template> ignores this option.
 I am sure this is a bug and hope it will be fixed eventually. I am afraid it will not be fixed
 soon, though.
 
-Meanwhile, C<Dist-Zilla-Role-TextTemplater> implements a workaround to let the option work, so C<TextTemplater>
+Meanwhile, C<TextTemplater> implements a workaround to let the option work, so C<TextTemplater>
 consumers can utilize the C<filename> option.
 
 =back
